//-------------------------------------------------------------------------------------------------
// TheGameInterfaceAbstract.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "TheGameInterfaceAbstract.h"
#include "engineAssert.h"
using namespace std;


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC TheGameInterfaceAbstract*	TheGameInterfaceAbstract::_theGameInterfaceAbstract = NULL;
STATIC bool							TheGameInterfaceAbstract::_isConstructed = false;



//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
TheGameInterfaceAbstract::~TheGameInterfaceAbstract()
{
	VALIDATE( _isConstructed );

	/// Set un-constructed
	_isConstructed = false;
}


//-------------------------------------------------------------------------------------------------
// Default Constructor
//-------------------------------------------------------------------------------------------------
TheGameInterfaceAbstract::TheGameInterfaceAbstract()
{
	/// Make sure the singleton object doesn't get constructed twice
	VALIDATE( !_isConstructed );

	/// Store a pointer to the derived game interface object being constructed
	_theGameInterfaceAbstract = this;

	/// Set construction complete
	_isConstructed = true;
}


//-------------------------------------------------------------------------------------------------
// ProcessCommandLine
// 
// Implement this function in the derived game class to handle (or change) command line options.
// 
// <cmdLine> is a string containing the application's command line as it was specified at
//	run time.  The game may feel free to parse and/or modify the contents of the string at
//	its discretion; the modified string will then be treated as the effective command line
//	by the application internally.
//-------------------------------------------------------------------------------------------------
VIRTUAL void TheGameInterfaceAbstract::ProcessCommandLine( string cmdLine )
{
	ASSERT( _isConstructed );
	ASSERT( _theGameInterfaceAbstract );

	UNUSED_ARG cmdLine;
}


//-------------------------------------------------------------------------------------------------
// Singleton
//-------------------------------------------------------------------------------------------------
STATIC TheGameInterfaceAbstract* TheGameInterfaceAbstract::Singleton()
{
	ASSERT( _isConstructed );
	ASSERT( _theGameInterfaceAbstract );

	return _theGameInterfaceAbstract;
}


//-------------------------------------------------------------------------------------------------
// PreInit
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::PreInit()
{
}


//-------------------------------------------------------------------------------------------------
// PostInit
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::PostInit()
{
}


//-------------------------------------------------------------------------------------------------
// Shutdown
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::Shutdown()
{
}


//-------------------------------------------------------------------------------------------------
// RunFrame
//-------------------------------------------------------------------------------------------------
void TheGameInterfaceAbstract::RunFrame()
{
}


//-------------------------------------------------------------------------------------------------
// ProcessInputEvent
//-------------------------------------------------------------------------------------------------
VIRTUAL const bool TheGameInterfaceAbstract::ProcessInputEvent( const TheInputSystem::inputEventType_e eventType, const byte_t key )
{
	ASSERT( _isConstructed );
	ASSERT( _theGameInterfaceAbstract );

	UNUSED_ARG eventType;
	UNUSED_ARG key;

	return false;
}


