//-------------------------------------------------------------------------------------------------
// TheInputSystem.h (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheInputSystem_
#define _include_TheInputSystem_


//-------------------------------------------------------------------------------------------------
// Easy-access macro
//-------------------------------------------------------------------------------------------------
#define INPUT (TheInputSystem::Singleton())


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheInputSystem
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheInputSystem
{
private:
	/// Static members
	static TheInputSystem* _theInputSystem;
	static bool _isConstructed;
	static bool _isInitialized;

	/// Private members
	bool	_isKeyDown[ 0xff ];
//	bool	_didKeyChangeStateThisFrame[ 0xff ];
	int		_mouseDeltaX;
	int		_mouseDeltaY;
	int		_mousePosX;
	int		_mousePosY;
//	int		_mouseLastKnownPosX;
//	int		_mouseLastKnownPosY;
	bool	_useWindowsMouseCursor;
	bool	_doesAppHaveFocus;

	/// Private constructors
	TheInputSystem(); // default constructor

public:
	enum inputEventType_e
	{
		INPUT_EVENT_NONE,
		INPUT_EVENT_KEYDOWN,
		INPUT_EVENT_KEYREPEAT,
		INPUT_EVENT_KEYUP,
		INPUT_EVENT_MOUSEWHEEL_UP,
		INPUT_EVENT_MOUSEWHEEL_DOWN,
	};

	/// Construction, initialization, and singleton methods
	~TheInputSystem(); // destructor

	static TheInputSystem* CreateSingleton();
	static TheInputSystem* Singleton();
	static void DestroySingleton();

	void Init();
	void Shutdown();

	/// Public methods
	void Update();
	void UseWindowsMouseCursor( const bool enable );
	const bool IsKeyDown( byte_t vKeyCode ) const;
	const bool ProcessInputEvent( const UINT messageID, const WPARAM wParam, const LPARAM lParam );
	void GetMousePosScreenAbsolute( int& screenMousePosX, int& screenMousePosY ) const;
	void SetMousePosScreenAbsolute( const int screenMousePosX, const int screenMousePosY );
	void GetMousePosClientAbsolute( int& clientMousePosX, int& clientMousePosY ) const;
	void SetMousePosClientAbsolute( const int clientMousePosX, const int clientMousePosY );
	void GetMouseDeltas( int& mouseDeltaX, int& mouseDeltaY );
	int GetMouseDeltaX();
	int GetMouseDeltaY();
	void OnAppGainFocus();
	void OnAppLoseFocus();
};


#endif // _include_TheInputSystem_
