//-------------------------------------------------------------------------------------------------
// TheVideoSystem.h (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheVideoSystem_
#define _include_TheVideoSystem_


//-------------------------------------------------------------------------------------------------
// Easy-access macro
//-------------------------------------------------------------------------------------------------
#define VIDEO (TheVideoSystem::Singleton())


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheVideoSystem
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheVideoSystem
{
public:
	/// Class constants
	static const DWORD
		WIN_STYLE_FLAGS_WINDOWED,
		WIN_STYLE_FLAGS_FULLSCREEN,
		WIN_STYLE_EX_FLAGS_WINDOWED,
		WIN_STYLE_EX_FLAGS_FULLSCREEN;

	enum
	{
		MAX_VID_MODES					= 1024,
	};

	/// Class substructures
	struct VidModeDesc
	{
		DEVMODE			devMode;
		unsigned int	flags;
	};

	/// Static member variables
	static TheVideoSystem* _theVideoSystem;
	static bool _isConstructed;
	static bool _isInitialized;
	static HWND _hWnd; // hWnd handle for the application's main window

	/// Private member variables
	const char* OPENGL_WINDOW_CLASS_NAME;

	int				_defaultWidth;
	int				_defaultHeight;
	int				_defaultBpp;
	int				_defaultFrequency;
	int				_defaultZBitDepth;
	unsigned int	_defaultFlags;
	VidModeDesc*	_vidModes[ MAX_VID_MODES ]; // FIXME: change to std::vector
	unsigned int	_numVidModes;
	unsigned int	_currentVidMode;
	RECT			_clientRect;
	POINT			_clientOffset;
	RECT			_windowRect;
	RECT			_desktopRect;
	HDC				_hDC; // display device context
	HGLRC			_hRC; // rendering context
	HWND			_hDesktopWnd; // hWnd window handle for the desktop
//	HWND			_hWnd; // hWnd handle for the application's main window
	bool			_showGDIStatusMessages; // if true, uses windows GDI to draw status messages

	/// Private member functions
	void _GetDesktopCharacteristics();
	const int _DetermineAvailableVideoModes();
	const int _CreateWindowedDisplayMode( const int width, const int height, const int bpp );
	void _CreateRenderingWindow();
	void _FillWindowClassDescription( WNDCLASS& wndClass );
	void _SetStyleFlagsForVidMode( DWORD& style, DWORD& style_ex, const int vidModeNum );
	void _SetClientAndWindowRectForVidMode( const int vidModeNum );
	void _FillPixelFormatDescriptor( PIXELFORMATDESCRIPTOR& pfd );
	void _CreateRenderingContextForWindow( const HWND hWnd, PIXELFORMATDESCRIPTOR& pixelFormatDesc );
	void _ClampWindowRectToDesktop( const bool forceResize );
	const int _SetModeClosestToDefaults();
	const int _GetVidModeClosestToProperties( const int width, const int height, const int bpp, const int frequency, const unsigned int flags );

	/// Private constructors
	TheVideoSystem();

	//---------------------------------------------------------------------------
	// Begin public class interface
	//---------------------------------------------------------------------------
public:
	enum
	{
		/// VidMode property flags
		VIDMODE_FLAG_FULLSCREEN			= BIT(  0 ),
//		VIDMODE_FLAG_DOUBLEBUFFERED		= BIT(  1 ),
	};

	/// Construction, initialization, and singleton methods
	~TheVideoSystem(); // destructor

	static TheVideoSystem* CreateSingleton();
	static TheVideoSystem* Singleton();
	static void DestroySingleton();

	void Init();
	void Shutdown();

	static const HWND GetHWnd();

	void UpdateGDIWaitPlaque( const char* const format, ... );
	inline const int GetCurrentVidMode() const { return _currentVidMode; }
	inline const int GetNumVidModes() const { return _numVidModes; }
	void QueryVidModeProperties( const int vidMode, int* pWidth, int* pHeight, int* pBpp, int* pFrequency, unsigned int* pFlags ) const;
	void QueryCurrentVidModeProperties( int* pWidth, int* pHeight, int* pBpp, int* pFrequency, unsigned int* pFlags ) const;
	const int GetCurrentVidModeWidth() const;
	const int GetCurrentVidModeHeight() const;
	void SetCurrentVidMode( const unsigned int vidMode, const bool forceVidRestart = false );
	const int SetCurrentVidModeByProperties( const int width, const int height, const int bpp, const int frequency, const unsigned int flags );
	void ListVidModes() const;
	void SwapBuffers() const;
	const bool IsVidModeWindowed( const int vidMode ) const;
	const bool IsCurrentVidModeWindowed() const;
	void SetDefaultVidModeProperties( const int width, const int height, const int bpp, const int frequency, const unsigned int flags );
//	const HWND GetHWnd() const;
	const std::string GetVidModeDescription( int vidMode ) const;
	void ConvertScreenCoordsToClientCoords( const int screenX, const int screenY, int& clientX, int& clientY );
	void ConvertClientCoordsToScreenCoords( const int clientX, const int clientY, int& screenX, int& screenY );
	inline HDC GetDisplayContext() const { return _hDC; }
	void NotifyOfWindowMove( int newClientX, int newClientY );
};



#endif // _include_TheVideoSystem_
