//-------------------------------------------------------------------------------------------------
// Timer.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "Timer.h"
#include "engineAssert.h"


//-------------------------------------------------------------------------------------------------
// Static member initialization
//-------------------------------------------------------------------------------------------------
STATIC double Timer::_queryPerfSecsPerCount = 0.0;


//-------------------------------------------------------------------------------------------------
// InitPerformanceCounter
//-------------------------------------------------------------------------------------------------
STATIC void Timer::_InitPerformanceCounter()
{
	/// Get the system's perf-ticks-per-second ratio
	LARGE_INTEGER queryPerfCountsPerSec;
	BOOL success = QueryPerformanceFrequency( &queryPerfCountsPerSec );

	/// Make sure the performance counter responded correctly
	VALIDATE( success );
	VALIDATE( queryPerfCountsPerSec.QuadPart );

	/// Calculate the seconds-per-tick ratio (inverse)
	_queryPerfSecsPerCount = (double) 1.0 / (double) queryPerfCountsPerSec.QuadPart;
}


//-------------------------------------------------------------------------------------------------
// GetSysTimeSeconds
//-------------------------------------------------------------------------------------------------
STATIC double Timer::GetSysTimeSeconds()
{
	/// Check if the perf timer needs initializing
	if( _queryPerfSecsPerCount == 0.0 )
	{
		_InitPerformanceCounter();
	}

	/// Calculate precise seconds using perf timer
	LARGE_INTEGER queryPerfCount;
	QueryPerformanceCounter( &queryPerfCount );
	double seconds = _queryPerfSecsPerCount * (double) queryPerfCount.QuadPart;

	return seconds;
}


//-------------------------------------------------------------------------------------------------
// GetSysTimeMS
//-------------------------------------------------------------------------------------------------
STATIC dword_t Timer::GetSysTimeMS()
{
	double ms = 1000.0 * GetSysTimeSeconds();
	return (dword_t) ms;
}


