//-------------------------------------------------------------------------------------------------
// engineAssert.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "engineAssert.h"
#include "TheApplication.h"
#include "TheVideoSystem.h"


//-------------------------------------------------------------------------------------------------
// Constants
//-------------------------------------------------------------------------------------------------
enum
{
	ASSERT_STRING_MAXLEN = 2048,

	MESSAGEBOX_FLAGS_WARNING	= ( MB_ICONWARNING | MB_SYSTEMMODAL | MB_SETFOREGROUND | MB_TOPMOST ),
	MESSAGEBOX_FLAGS_ERROR		= ( MB_ICONERROR | MB_SYSTEMMODAL | MB_SETFOREGROUND | MB_TOPMOST ),
};


//-------------------------------------------------------------------------------------------------
// UserBreakpoint
//-------------------------------------------------------------------------------------------------
typedef BOOL (CALLBACK isDebugFunc_f)();
void UserBreakpoint()
{
#if defined (ENABLE_USER_BREAKPOINTS)

    HINSTANCE		hInst;

	/// Get a handle to KERNEL32.DLL
    hInst = GetModuleHandle( TEXT( "KERNEL32" ) );

	/// Exit if unable to get a handle to KERNEL32.DLL
	if( !hInst )
		return;

	/// Create an MsDev user breakpoint for easier debugging if ENABLE_USER_BREAKPOINTS is enabled
	isDebugFunc_f*	isDebuggerPresentFunc;

	/// Get a handle to IsDebuggerPresent() in KERNEL32.DLL
	isDebuggerPresentFunc = (isDebugFunc_f*) GetProcAddress( hInst, "IsDebuggerPresent" );

	/// Exit if unable to find IsDebuggerPresent() in KERNEL32.DLL
	if( !isDebuggerPresentFunc )
		return;

	/// Exit if no debugger present
	if( !isDebuggerPresentFunc() )
		return;

	/// Create an MsDev user breakpoint
	__asm int 3;

#endif // #if defined (ENABLE_USER_BREAKPOINTS)
}


//-------------------------------------------------------------------------------------------------
// AssertionFailure
//-------------------------------------------------------------------------------------------------
__declspec( noreturn )
void AssertionFailure( const int line, const char* const fileName, const char* const assertType, const char* const expression, const char* const format, ... )
{
	va_list			args;
	char			text_message[ ASSERT_STRING_MAXLEN ]; // the variable-argument print-formatted string
	char			text_final[ ASSERT_STRING_MAXLEN ]; // <text_message> with line & file info added
	char			msgBoxCaption[ ASSERT_STRING_MAXLEN ];
	HWND			hWnd = NULL;
	static bool		assertionInProgress = false;

	/// Be careful not to cause another assertion failure within this function
	if( !assertionInProgress )
	{
		assertionInProgress = true;

		/// Write formatted string into <text_message>
		va_start( args, format );
		_vsnprintf( text_message, ASSERT_STRING_MAXLEN, format, args );
		va_end( args );
		text_message[ ASSERT_STRING_MAXLEN - 1 ] = '\0';

		/// Write line & filename info as well as the previous <text_message> into <text_final>
		Sprintf( text_final, ASSERT_STRING_MAXLEN, "ASSERTION FAILURE:\n%s(%d):  (%s)\nExpression was: ( %s )\n\n   %s\n", fileName, line, assertType, expression, text_message );

		/// Send a message to the debug console
		DebugPrintf( text_final );

		/// Restore the desktop
		ChangeDisplaySettings( NULL, 0 );

		/// Attempt to get a window handle for opening message boxes, etc.
//		VIDEO->GetHWnd();
		hWnd = TheVideoSystem::GetHWnd();

		/// Hide the application window and open a message box
		ShowWindow( hWnd, SW_HIDE ); // hide the window (forever!)

		// always show the error, even when exiting
		Sprintf( msgBoxCaption, ASSERT_STRING_MAXLEN, "%s : Program Error", APPLICATION->GetName().c_str() );
		MessageBox( hWnd, text_final, msgBoxCaption, MB_OK | MESSAGEBOX_FLAGS_ERROR );

		/// Create an MsDev user breakpoint if running in a debugger context
		UserBreakpoint();

		/// Print final "shutting down" message
		Sprintf( msgBoxCaption, ASSERT_STRING_MAXLEN, "%s : Emergency Shutdown", APPLICATION->GetName().c_str() );
		MessageBox( hWnd, "The application will now close.", msgBoxCaption, MB_OK | MESSAGEBOX_FLAGS_ERROR );

		/// Shut down the application
		APPLICATION->RequestShutdown();
		APPLICATION->CheckForShutdownRequest();
		APPLICATION->DestroySingleton();
	}

	/// Exit forcibly (we CANNOT return from this function!)
	exit( TheApplication::EXIT_MESSAGE_ERROR_GENERIC );
}


