//-------------------------------------------------------------------------------------------------
// engineCommon.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "engineCommon.h"
#include "engineAssert.h"



//-------------------------------------------------------------------------------------------------
// ::Sprintf (like the standard C library one, only length-overrun-safe)
//-------------------------------------------------------------------------------------------------
int Sprintf( char* buffer, size_t maxLength, const char* format, ... )
{
	va_list		args;
	int			return_value;

	ASSERT( buffer );
	ASSERT( format );
	ASSERT_MSG2( maxLength > 0, "Bad length %i specified for Sprintf; format string was \"%s\"", maxLength, format );

	/// Write formatted string into <buffer>
	va_start( args, format );
	return_value = _vsnprintf( buffer, maxLength, format, args );
	va_end( args );
	buffer[ maxLength - 1 ] = '\0';

	/// Follow the same return-value convention as _vsnprintf()
	return( return_value );
}


//-------------------------------------------------------------------------------------------------
// ::DebugPrintf (like OutputDebugString, only with sprintf-style formatting)
//-------------------------------------------------------------------------------------------------
void DebugPrintf( const char* format, ... )
{
	const unsigned int	TEXT_BUFFER_SIZE = 2048;
	char				text[ TEXT_BUFFER_SIZE ];
	va_list				args;

	ASSERT( format );

	/// Write formatted string into <buffer>
	va_start( args, format );
	_vsnprintf( text, TEXT_BUFFER_SIZE, format, args );
	va_end( args );
	text[ TEXT_BUFFER_SIZE - 1 ] = '\0';

	OutputDebugString( text );
}


