// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#pragma once
#include "BaseNode.h"
#include "BaseGrowthHeuristic.h"
#include "Projectiles.h"
#include "Spawner.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Base
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class Base
{
private:
	Base(); // not allowed
public:
	static const Rgba COLOR_TERMINATOR_LIGHT_EMPTY;
	static const Rgba COLOR_TERMINATOR_DARK_EMPTY;
	static const Rgba COLOR_TERMINATOR_LIGHT_JUNCTION;
	static const Rgba COLOR_TERMINATOR_DARK_JUNCTION;
	static const Rgba COLOR_TERMINATOR_LIGHT_TERMINATOR;
	static const Rgba COLOR_TERMINATOR_DARK_TERMINATOR;
	static const Rgba COLOR_TERMINATOR_LIGHT_HEART;
	static const Rgba COLOR_TERMINATOR_DARK_HEART;
	static const Rgba COLOR_CONNECTOR_LIGHT;
	static const Rgba COLOR_CONNECTOR_DARK;

	Vector2									m_pos;
	int										m_width;
	int										m_height;
	std::vector< std::vector<BaseNode> >	m_grid;
	bool									m_isAlive;
	bool									m_isDyingFromHeart;
	AABB2									m_bounds;
	float									m_outerRadius;
	int										m_heartX;
	int										m_heartY;
	double									m_timeOfDeath;

	/// Base.cpp
	Base( const Vector2& position, int gridWidth, int gridHeight );
	~Base();
	void InitBounds();
	void InitNodes();
	void AddHeart( int heart_x, int heart_y );
	void AddConnection( int start_x, int start_y, int end_x, int end_y );
	static Base& SpawnBase( const Vector2& position, int gridWidth, int gridHeight );
	static void ClearBaseList();
	void Update();
	void UpdateConnectionForNodeDirection( BaseNode& node, HexDirection direction );
	void UpdateConnectionsForNode( BaseNode& node );
	bool UpdatePipe( BasePipe& pipe );
	void ContinueChainReactionAtNode( BaseNode& node );
	void ResolveNodeDeath( BaseNode& node );
	void OnHeartDeath();
	void ReserveGridNode( int x, int y );
	void Generate( const BaseGrowthHeuristic& heuristic = BaseGrowthHeuristic::DEFAULT_HEURISTIC );
	void GenerateUsingTestHeuristic( const BaseGrowthHeuristic& heuristic );
	void GenerateStuffAtNode( int x, int y, const BaseGrowthHeuristic& heuristic );
	float ScoreConnectionDirectionFromXY( HexDirection dir, int x, int y, const BaseGrowthHeuristic& heuristic );
	void AddSpawner( int spawner_x, int spawner_y, SpawnType spawnType = OBJTYPE_BULLETSMALL, double spawnInterval = SPAWNINTERVAL_DEFAULT, int maxSpawnCount = SPAWNMAXCOUNT_DEFAULT );
	BaseNode* GetRandomOccupiedNode();
	float CalcAnimFraction();
	float CalcPrevAnimFraction();
	void UpdateShockwave();
	void CheckNodeVersesShockwave( BaseNode& node, float minFractionDead, float maxFractionDead );


	/// Base_Draw.cpp
	Rgba GetLightColorForNodeType( BaseNodeType type );
	Rgba GetDarkColorForNodeType( BaseNodeType type );
	void DrawNode( const Vector2& centerPos, BaseNodeType type );
	void DrawPipe( BasePipe& pipe );
	void DrawExplosionAtPoint( const Vector2& point );
	void Draw();
	void DrawConnectionsForGridXY( int x, int y );
	void DrawShockwave( float fractionDead );
};


//-------------------------------------------------------------------------------------------------
// Extern'd globals
//-------------------------------------------------------------------------------------------------
extern std::list<Base*> baseList;


