// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.h"
#include "BaseCommon.h"


double g_timeNow = 0.0;
double g_frameSeconds = 0.01;


//-------------------------------------------------------------------------------------------------
// CalcDisplacementForGridXY
//-------------------------------------------------------------------------------------------------
Vector2 CalcOffsetForGridXY( int x, int y )
{
	Vector2 offset = (float) x * GRID_BASIS_WIDTH + (float) y * GRID_BASIS_HEIGHT;
	return offset;
}


//-------------------------------------------------------------------------------------------------
// CalcOffsetForDirection
//-------------------------------------------------------------------------------------------------
Vector2 CalcOffsetForDirection( HexDirection direction )
{
	int x, y;
	GetXYFromDirection( direction, x, y );
	return CalcOffsetForGridXY( x, y );
}


//-------------------------------------------------------------------------------------------------
// GetDirectionFromXY
//-------------------------------------------------------------------------------------------------
HexDirection GetDirectionFromXY( int x, int y )
{
	if( x == 1 && y == 0 )
	{
		return HEXDIRECTION_EAST;
	}
	else if( x == -1 && y == 0 )
	{
		return HEXDIRECTION_WEST;
	}
	else if( x == 0 && y == 1 )
	{
		return HEXDIRECTION_NORTHEAST;
	}
	else if( x == 0 && y == -1 )
	{
		return HEXDIRECTION_SOUTHWEST;
	}
	else if( x == -1 && y == 1 )
	{
		return HEXDIRECTION_NORTHWEST;
	}
	else if( x == 1 && y == -1 )
	{
		return HEXDIRECTION_SOUTHEAST;
	}

	PANIC_MSG2( "GetDirectionFromXY : couldn't find adjacent direction for (%d,%d)", x, y );
}


//-------------------------------------------------------------------------------------------------
// GetXYFromDirection
//-------------------------------------------------------------------------------------------------
void GetXYFromDirection( const HexDirection dir, int& x, int& y )
{
	if( dir == HEXDIRECTION_EAST )
	{
		x = 1;
		y = 0;
	}
	else if( dir == HEXDIRECTION_WEST )
	{
		x = -1;
		y = 0;
	}
	else if( dir == HEXDIRECTION_NORTHEAST )
	{
		x = 0;
		y = 1;
	}
	else if( dir == HEXDIRECTION_SOUTHWEST )
	{
		x = 0;
		y = -1;
	}
	else if( dir == HEXDIRECTION_NORTHWEST )
	{
		x = -1;
		y = 1;
	}
	else if( dir == HEXDIRECTION_SOUTHEAST )
	{
		x = 1;
		y = -1;
	}
	else
	{
		PANIC_MSG1( "GetXYFromDirection : unknown direction # %d", dir );
	}
}


//-------------------------------------------------------------------------------------------------
// ReverseDirection
//-------------------------------------------------------------------------------------------------
HexDirection ReverseDirection( const HexDirection dir )
{
	switch( dir )
	{
		case HEXDIRECTION_EAST:		return HEXDIRECTION_WEST;			break;
		case HEXDIRECTION_WEST:		return HEXDIRECTION_EAST;			break;
		case HEXDIRECTION_NORTHEAST:	return HEXDIRECTION_SOUTHWEST;		break;
		case HEXDIRECTION_NORTHWEST:	return HEXDIRECTION_SOUTHEAST;		break;
		case HEXDIRECTION_SOUTHEAST:	return HEXDIRECTION_NORTHWEST;		break;
		case HEXDIRECTION_SOUTHWEST:	return HEXDIRECTION_NORTHEAST;		break;
		default:
			PANIC_MSG1( "ReverseDirection : bad dir #%d!\n", (int) dir ); 
			break;
	}
}


//-------------------------------------------------------------------------------------------------
// CalcGridDistBetweenPoints
//-------------------------------------------------------------------------------------------------
int CalcGridDistBetweenPoints( int x1, int y1, int x2, int y2 )
{
	int dispX = x2 - x1;
	int dispY = y2 - y1;

	/// Check if dispX and dispY have opposite signs
	if( (dispX < 0) == (dispY > 0) )
	{
		return Min( abs( dispX ), abs( dispY ) );
	}
	else
	{
		return abs( dispX ) + abs( dispY );
	}
}



