// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#pragma once
#include "gameCommon.h"
#include "GameBalance.h"


enum HexDirection
{
	/// Six directions in a hex grid
	HEXDIRECTION_NONE = -1,
	HEXDIRECTION_EAST = 0,
	HEXDIRECTION_WEST,
	HEXDIRECTION_NORTHEAST,
	HEXDIRECTION_NORTHWEST,
	HEXDIRECTION_SOUTHEAST,
	HEXDIRECTION_SOUTHWEST,
	NUM_HEXDIRECTIONS,
};


enum BaseNodeType
{
	NODETYPE_OUTOFBOUNDS,	// Node is outside the set of those used by the base
	NODETYPE_EMPTY,			// Node is either unpopulated or has been destroyed
	NODETYPE_JUNCTION,		// Node represents a junction of two or more pipes
	NODETYPE_TERMINATOR,	// Node is the destructible end of pipe
	NODETYPE_HEART,			// The (a?) heart of the base; kill this for a chain reaction
};


enum PipeStatus
{
//	PIPESTATUS_EMPTY,
	PIPESTATUS_INTACT,			// Pipe is intact
	PIPESTATUS_DYING_FORWARD,	// Pipe is currently dying, from <startNode> to <endNode>
	PIPESTATUS_DYING_BACKWARD,	// Pipe is currently dying, from <endNode> to <startNode>
};




//-------------------------------------------------------------------------------------------------
// Forward class declarations
//-------------------------------------------------------------------------------------------------
struct BaseNode;
struct BasePipe;
class Base;


//-------------------------------------------------------------------------------------------------
// Functions exported from BaseCommon.cpp
//-------------------------------------------------------------------------------------------------
Vector2 CalcOffsetForGridXY( int x, int y );
Vector2 CalcOffsetForDirection( HexDirection direction );
HexDirection GetDirectionFromXY( int x, int y );
void GetXYFromDirection( const HexDirection dir, int& x, int& y );
HexDirection ReverseDirection( const HexDirection dir );
int CalcGridDistBetweenPoints( int x1, int y1, int x2, int y2 );


//-------------------------------------------------------------------------------------------------
// Globals
//-------------------------------------------------------------------------------------------------
extern double g_timeNow;
extern double g_frameSeconds;

