// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.h"
#include "BaseNode.h"
#include "Image.h"
#include "gameCommon.h"
#include "TheGame.h"
using namespace std;


//-------------------------------------------------------------------------------------------------
// BaseNode (default constructor)
//-------------------------------------------------------------------------------------------------
BaseNode::BaseNode()
{
}


//-------------------------------------------------------------------------------------------------
// ~BaseNode (destructor)
//-------------------------------------------------------------------------------------------------
BaseNode::~BaseNode()
{
}


//-------------------------------------------------------------------------------------------------
// InitVacantNode
//-------------------------------------------------------------------------------------------------
void BaseNode::InitVacantNode( Base* pBase, int x, int y )
{
	m_type = NODETYPE_OUTOFBOUNDS; // This node can't be used until reserved
	m_numberOfConnections = 0;
	m_gridX = x;
	m_gridY = y;
	m_base = pBase;
	m_hitPoints = TERMINATOR_HP;
	m_spawner = NULL;
	m_isConnectedToHeart = false;

	for( int i = 0; i < NUM_HEXDIRECTIONS; ++ i )
	{
		m_connections[ i ] = NULL;
	}

	m_pos = m_base->m_pos + CalcOffsetForGridXY( x, y );
	Vector2 radiusBounds( NODE_RADIUS, NODE_RADIUS );
	m_bounds.SetFromPointPadding( m_pos, radiusBounds );
}


//-------------------------------------------------------------------------------------------------
// FindPipeAmongConnections
//-------------------------------------------------------------------------------------------------
HexDirection BaseNode::FindPipeAmongConnections( BasePipe* pipe )
{
	for( int i = 0; i < NUM_HEXDIRECTIONS; ++ i )
	{
		if( m_connections[ i ] == pipe )
		{
			return (HexDirection) i;
		}
	}

	return HEXDIRECTION_NONE;
}


//-------------------------------------------------------------------------------------------------
// UnhookPipe
//-------------------------------------------------------------------------------------------------
void BaseNode::UnhookPipe( BasePipe* pipe )
{
	for( int i = 0; i < NUM_HEXDIRECTIONS; ++ i )
	{
		if( m_connections[ i ] == pipe )
		{
			m_connections[ i ] = NULL;
			m_numberOfConnections --;
			return;
		}
	}
}


//-------------------------------------------------------------------------------------------------
// UpgradeConnectionType
//-------------------------------------------------------------------------------------------------
void BaseNode::UpgradeConnectionType()
{
	switch( m_type )
	{
		case NODETYPE_OUTOFBOUNDS:
//			PANIC_MSG4( "UpgradeConnectionType : node at grid %d,%d (world %.2f, %.2f) was NODETYPE_OUTOFBOUNDS!", gridX, gridY, pos.x, pos.y );
//			break;
// for now, allow this to fall through and build it anyway
		case NODETYPE_EMPTY:
			m_type = NODETYPE_TERMINATOR;
			break;
		case NODETYPE_TERMINATOR:
		case NODETYPE_JUNCTION:
			m_type = NODETYPE_JUNCTION;
			break;
		case NODETYPE_HEART:
			m_type = NODETYPE_HEART;
			break;
	}
}


//-------------------------------------------------------------------------------------------------
// IsEmpty
//-------------------------------------------------------------------------------------------------
bool BaseNode::IsEmpty()
{
	return (m_type == NODETYPE_EMPTY) || (m_type == NODETYPE_OUTOFBOUNDS);
}


//-------------------------------------------------------------------------------------------------
// UpdateSpawner
//-------------------------------------------------------------------------------------------------
void BaseNode::UpdateSpawner()
{
	/// Do nothing if no spawner present at this node
	if( !m_spawner )
		return;

	m_spawner->Update( m_pos, Vector2::ZERO );
}

