// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.h"
#include "Enemy.h"
#include "Player.h"
#include "gameCommon.h"
#include "BaseCommon.h"
#include "GameBalance.h"
#include "TheGame.h"
using namespace std;

STATIC list< Enemy* > Enemy::enemyList;
STATIC list< EnemyGrunt* > EnemyGrunt::enemyGruntList;


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Enemy - base class for all enemy types
//
///////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
// Enemy::Enemy
//-------------------------------------------------------------------------------------------------
Enemy::Enemy()
{
	enemyList.push_back( this );
}


//-------------------------------------------------------------------------------------------------
// Enemy::~Enemy
//-------------------------------------------------------------------------------------------------
VIRTUAL Enemy::~Enemy()
{
}


//-------------------------------------------------------------------------------------------------
// Enemy::UpdateEnemies
//-------------------------------------------------------------------------------------------------
STATIC Enemy::UpdateEnemies()
{
	int count = enemyList.size();
	int count2 = EnemyGrunt::enemyGruntList.size();
	RENDERER->XYPrintf( g_player.m_pos.x, g_player.m_pos.y, "Total of %d enemies", count );
	RENDERER->XYPrintf( g_player.m_pos.x, g_player.m_pos.y + 100.0f, "Total of %d grunts", count2 );

	std::list< Enemy* >::iterator iter;
	for( iter = enemyList.begin(); iter != enemyList.end(); ++ iter )
	{
		Enemy* pEnemy = *iter;
		pEnemy->Update();
	}
}


//-------------------------------------------------------------------------------------------------
// Enemy::DrawEnemies
//-------------------------------------------------------------------------------------------------
STATIC Enemy::DrawEnemies()
{
	std::list< Enemy* >::iterator iter;
	for( iter = enemyList.begin(); iter != enemyList.end(); ++ iter )
	{
		Enemy* pEnemy = *iter;
		pEnemy->Draw();
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		EnemyGrunt
//
///////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
// EnemyGrunt::EnemyGrunt
//-------------------------------------------------------------------------------------------------
EnemyGrunt::EnemyGrunt( const Vector2& pos, const Vector2& vel )
:
m_timeLastShot( g_timeNow + ENEMY_GRUNT_WEAPONS_ARM_TIME )
{
	enemyGruntList.push_back( this );

	m_pos = pos;

	/// Launch direction is perpendicular to player
	Vector2 vecToPlayer = g_player.m_pos - m_pos;
	m_facingRadians = vecToPlayer.GetRadians() + DEG2RAD( 160.0f );
	m_forward.SetFromCosSinRadians( m_facingRadians );
	m_vel = m_forward * ENEMY_GRUNT_LAUNCHSPEED;
}


//-------------------------------------------------------------------------------------------------
// EnemyGrunt::EnemyGrunt
//-------------------------------------------------------------------------------------------------
EnemyGrunt::EnemyGrunt()
:
m_timeLastShot( g_timeNow + ENEMY_GRUNT_WEAPONS_ARM_TIME )
{
}


//-------------------------------------------------------------------------------------------------
// EnemyGrunt::~EnemyGrunt
//-------------------------------------------------------------------------------------------------
EnemyGrunt::~EnemyGrunt()
{
}


//-------------------------------------------------------------------------------------------------
// EnemyGrunt::Update
//-------------------------------------------------------------------------------------------------
void EnemyGrunt::Update()
{
	float timeMultiplier = (float) g_frameSeconds;

	float preferredDistance = ENEMY_GRUNT_PREFERRED_DISTANCE;

	/// Locate goal position
	Vector2 vecFromPlayer = m_pos - g_player.m_pos;
	float radians = vecFromPlayer.GetRadians();
	radians += (float) PI * 0.01f;
	Vector2 goal;
	goal.SetFromRadiusCosSinRadians( preferredDistance, radians );
	goal += g_player.m_pos;

	/// Calculate normalized acceleration towards goal
	if( g_debugDrawCollisions )
	{
		RENDERER->DrawVector( m_pos, goal, Rgba::GREEN, Rgba::RED );
	}
	Vector2 vecToGoal = goal - m_pos;
	float dist = vecToGoal.Normalize();
	Vector2 accel = vecToGoal * ENEMY_GRUNT_ACCELERATION * timeMultiplier;

	/// Accelerate and move the object
	m_vel += accel;
	m_vel.SetLength( ENEMY_GRUNT_LAUNCHSPEED ); // lock them at a fixed velocity
	Vector2 moveVector = m_vel * timeMultiplier;
	m_pos += moveVector;

	/// Check if this grunt should shoot a bullet
	double timeSinceLastShot = g_timeNow - m_timeLastShot;
	if( timeSinceLastShot > ENEMY_GRUNT_BULLET_FIRE_INTERVAL )
	{
		m_timeLastShot = g_timeNow;
		new EnemyBulletSmall( m_pos, Vector2::ZERO );
	}
}


//-------------------------------------------------------------------------------------------------
// EnemyGrunt::Draw
//-------------------------------------------------------------------------------------------------
void EnemyGrunt::Draw()
{
	glEnable( GL_POINT_SMOOTH );
	glPointSize( 10.0f );
	RENDERER->SetColor( Rgba::GREEN );
	glBegin( GL_POINTS );
	{
		glVertex2f( m_pos.x, m_pos.y );
	}
	glEnd(); // GL_POINTS

	RENDERER->SetColor( 0.05f, 0.5f, 0.1f, 0.1f );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );
	glBegin( GL_LINES );
	{
		glVertex2f( m_pos.x, m_pos.y );
		glVertex2f( g_player.m_pos.x, g_player.m_pos.y );
	}
	glEnd(); // GL_LINES
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		EnemyBulletSmall
//
///////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
// EnemyBulletSmall::EnemyBulletSmall
//-------------------------------------------------------------------------------------------------
EnemyBulletSmall::EnemyBulletSmall( const Vector2& pos, const Vector2& vel )
{
	m_pos = pos;

	/// First launch direction is straight towards player
	Vector2 vecToPlayer = g_player.m_pos - m_pos;
	m_facingRadians = vecToPlayer.GetRadians();
	m_forward.SetFromCosSinRadians( m_facingRadians );
	m_vel = m_forward * ENEMY_BULLETSMALL_LAUNCHSPEED;

	/// Second launch direction is straight towards where player 
}


//-------------------------------------------------------------------------------------------------
// EnemyBulletSmall::EnemyBulletSmall
//-------------------------------------------------------------------------------------------------
EnemyBulletSmall::EnemyBulletSmall()
{
}


 //-------------------------------------------------------------------------------------------------
// EnemyBulletSmall::~EnemyBulletSmall
//-------------------------------------------------------------------------------------------------
EnemyBulletSmall::~EnemyBulletSmall()
{
}


//-------------------------------------------------------------------------------------------------
// EnemyBulletSmall::Update
//-------------------------------------------------------------------------------------------------
void EnemyBulletSmall::Update()
{
	float timeMultiplier = (float) g_frameSeconds;

	Vector2 moveVector = m_vel * timeMultiplier;
	m_pos += moveVector;
}


//-------------------------------------------------------------------------------------------------
// EnemyBulletSmall::Draw
//-------------------------------------------------------------------------------------------------
void EnemyBulletSmall::Draw()
{
	glEnable( GL_POINT_SMOOTH );
	glPointSize( 5.0f );
	RENDERER->SetColor( Rgba::RED );
	glBegin( GL_POINTS );
	{
		glVertex2f( m_pos.x, m_pos.y );
	}
	glEnd(); // GL_POINTS

	RENDERER->SetColor( 0.5f, 0.1f, 0.05f, 0.1f );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );
	glBegin( GL_LINES );
	{
		glVertex2f( m_pos.x, m_pos.y );
		glVertex2f( g_player.m_pos.x, g_player.m_pos.y );
	}
	glEnd(); // GL_LINES
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
}


