// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.h"
#include "ParticleEmitter.h"
#include "Image.h"

//-------------------------------------------------------------------------------------------------
ParticleEmitter::ParticleEmitter( const word_t particleImageId, const word_t maxParticlesActive )
:	m_creationTimeMS			( GetTickCount() ),
	m_maxActiveParticles	( maxParticlesActive ),
	m_particles				( maxParticlesActive ),
	m_particleImageId		( particleImageId )
{
	ASSERT( m_maxActiveParticles > 0 );
}

//-------------------------------------------------------------------------------------------------
ParticleEmitter::~ParticleEmitter()
{
}

//-------------------------------------------------------------------------------------------------
void ParticleEmitter::Render()
{
	glEnable( GL_TEXTURE_2D );

	Image& particleImage = Image::Get( m_particleImageId );
	RENDERER->SetCurrentTexture( particleImage.GetTexNum() );

	for( ActiveParticleList::iterator i = m_activeParticles.begin(); i != m_activeParticles.end(); ++i )
	{
		Particle* p = *i;
		ASSERT( p->isActive );
		particleImage.Draw( p->renderPosition, p->renderScale, p->renderColor, false );
	}

	glDisable( GL_TEXTURE_2D );
}

//-------------------------------------------------------------------------------------------------
ParticleEmitter_Test::ParticleEmitter_Test()
:	ParticleEmitter( IMAGE_PARTICLE_TEST, 20000 )	
{
	const float oocount = 1.0f / (float) m_maxActiveParticles;
	for( int i = 0; i < m_maxActiveParticles; i++ )
	{
		Particle& p = m_particles[ i ];
		Activate( &p );
		p.renderColor.SetRGBA( 1.0f, 1.0f, 1.0f, 0.5f + ((i/2) * oocount) );
		p.renderPosition.SetFromCosSinDegrees( 360 * i * oocount );
		p.renderPosition.x += 0.5;
		p.renderPosition.y += 0.5;
		p.renderPosition *= 100;
		p.renderScale = 1.0f;
	}
}

//-------------------------------------------------------------------------------------------------
extern float testVoob;
void ParticleEmitter_Test::Update()
{
	static int turn = 0;
	const float oocount = 1.0f / (float) m_maxActiveParticles;
	int count = 0;
	for( ActiveParticleList::iterator i = m_activeParticles.begin(); i != m_activeParticles.end(); ++i )
	{
		Particle* p = *i;
		ASSERT( p->isActive );
		
		p->renderPosition.SetFromCosSinDegrees( (turn + (360 * count * oocount)) * 20.0f );
		p->renderPosition *= count * 0.03f * testVoob;
		count++;
	}
	turn -= 2;
}