// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#pragma once

//-------------------------------------------------------------------------------------------------
#include "gameSysHeaders.h"
#include "engineAssert.h"
#include "Vector2.h"
#include "Rgba.h"

//-------------------------------------------------------------------------------------------------
struct Particle // inherit from this to keep user data in your ParticleEmitter-derived classes
{
	bool	isActive;
	Vector2 renderPosition;
	Rgba	renderColor;
	float	renderScale;

	// constructor to initialize data to usable values
	Particle() 
	:	isActive		( false ),
		renderPosition	( 0.0f, 0.0f ), 
		renderColor		( 1.0f, 1.0f, 1.0f ), 
		renderScale		( 1.0f ) 
	{ 
		(void)0;
	}

	// copy constructor
	Particle( const Particle& copyme )
	{
		isActive		= copyme.isActive;
		renderPosition	= copyme.renderPosition;
		renderColor		= copyme.renderColor;
		renderScale		= copyme.renderScale;
	}
};

//-------------------------------------------------------------------------------------------------
class ParticleEmitter
{
protected:

	typedef std::list< Particle* > ActiveParticleList;
	typedef std::vector< Particle > ParticleVector;
	dword_t					m_creationTimeMS;
	word_t					m_maxActiveParticles;
	ParticleVector			m_particles;
	ActiveParticleList		m_activeParticles; // points into m_particles
	dword_t					m_particleImageId;

	inline void Activate( Particle* const p );
	inline void Deactivate( Particle* const p );

public:

	ParticleEmitter( const word_t particleImageId, const word_t maxParticlesActive = 100 );
	virtual ~ParticleEmitter();

	virtual void Update() = 0;
	virtual void Render();

};

//-------------------------------------------------------------------------------------------------
inline void ParticleEmitter::Activate( Particle* const p )
{
	ASSERT( p );
	if( !p->isActive )
	{
		p->isActive = true;
		m_activeParticles.push_back( p );
	}
}

//-------------------------------------------------------------------------------------------------
inline void ParticleEmitter::Deactivate( Particle* const p )
{
	ASSERT( p );
	if( p->isActive )
	{
		ActiveParticleList::iterator i = std::find( m_activeParticles.begin(), m_activeParticles.end(), p );
		ASSERT( i != m_activeParticles.end() );

		(*i)->isActive = false;
		m_activeParticles.erase( i );
	}
}

//-------------------------------------------------------------------------------------------------
class ParticleEmitter_Test : public ParticleEmitter // FIXME: remove this, it's just for testing
{
public:
	ParticleEmitter_Test();
	virtual ~ParticleEmitter_Test(){}
	virtual void Update();
};

