// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.h"
#include "Player.h"
#include "Image.h"
#include "gameCommon.h"
#include "TheGame.h"


//-------------------------------------------------------------------------------------------------
// Globals
//-------------------------------------------------------------------------------------------------
Player g_player;



//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Player::Player()
:
m_pos( Vector2::ZERO ),
m_vel( Vector2::ZERO ),
m_forward( 1.0f, 0.0f ),
m_facingRadians( 0.0f ),
m_shipImageScale( 0.8f ),
m_isKeyDown_Up( false ),
m_isKeyDown_Down( false ),
m_isKeyDown_Left( false ),
m_isKeyDown_Right( false )
{
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Player::~Player()
{
}


//-------------------------------------------------------------------------------------------------
// Init
//-------------------------------------------------------------------------------------------------
void Player::Init()
{
}


//-------------------------------------------------------------------------------------------------
// Update
//-------------------------------------------------------------------------------------------------
void Player::Update()
{
	/// Adjust ship's heading based on x-axis mouse movement
	float turnDeltaRadians = -0.01f * (float) INPUT->GetMouseDeltaX() * g_game->m_mouseSensitivity.x;
	turnDeltaRadians *= g_game->m_shipTurnRate;
	m_facingRadians += turnDeltaRadians;

	/// Update forward vector
	m_forward.SetFromCosSinRadians( m_facingRadians );

	/// Generate right vector
	Vector2 right( m_forward.y, -m_forward.x );

	/// Calculate the move vector for this frame
	const float moveSpeed = PLAYER_MOVE_SPEED;
	Vector2 move( Vector2::ZERO );

	if( g_controlSchemeType == CONTROL_SCHEME_XKOBO )
	{
		m_forward = GetNormalVectorForMoveDirection( m_direction );
		right = Vector2( m_forward.y, -m_forward.x );
		move = moveSpeed * m_forward;
	}
	else if( g_controlSchemeType == CONTROL_SCHEME_SHIPRELATIVEROTATION )
	{
		if( m_isKeyDown_Up )
			move += m_forward * moveSpeed;

		if( m_isKeyDown_Down )
			move -= m_forward * moveSpeed;

		if( m_isKeyDown_Right )
			move += right * moveSpeed;

		if( m_isKeyDown_Left )
			move -= right * moveSpeed;
	}

	m_vel = move;

	/// Apply the move vector to the player's position
	m_pos += move * (float) g_frameSeconds;

	/// Check if the player is shooting
	if( g_playerIsShooting )
	{
		double secondsPerBullet = 1.0 / (double) BULLETS_PER_SECOND;
		static double bulletTimeAccumulated = 0.0;

		double frameTime = g_frameSeconds;
		bulletTimeAccumulated += frameTime;
		int numBullets = (int)( bulletTimeAccumulated * (double) BULLETS_PER_SECOND );
		if( numBullets > 0 )
		{
			double bulletTimeSpent = secondsPerBullet * (double) numBullets;
			bulletTimeAccumulated -= bulletTimeSpent;

			Vector2 shootVector = g_player.m_forward;
			shootVector *= PLAYER_BULLET_SPEED;

			Projectile::SpawnProjectile( g_player.m_pos, shootVector, PROJECTILETYPE_PLAYER_BOLT, (float) numBullets );

			if( numBullets > 1 )
			{
				testVoob = testVoob;
			}
		}
	}
}


//-------------------------------------------------------------------------------------------------
// GetNormalVectorForMoveDirection
//-------------------------------------------------------------------------------------------------
Vector2 Player::GetNormalVectorForMoveDirection( OctDirection direction )
{
	const float EDGE = 1.0f;
	const float DIAG = 0.70710678118654752440084436210485f;
	const float ZERO = 0.0f;

	int modifiedDirection = direction - 2;
	if( modifiedDirection < OCTDIRECTION_EAST )
		modifiedDirection += NUM_OCTDIRECTIONS;

	if( modifiedDirection == OCTDIRECTION_EAST )		return Vector2( +EDGE, +ZERO );
	if( modifiedDirection == OCTDIRECTION_NORTHEAST )	return Vector2( +DIAG, +DIAG );
	if( modifiedDirection == OCTDIRECTION_NORTH )		return Vector2( +ZERO, +EDGE );
	if( modifiedDirection == OCTDIRECTION_NORTHWEST )	return Vector2( -DIAG, +DIAG );
	if( modifiedDirection == OCTDIRECTION_WEST )		return Vector2( -EDGE, +ZERO );
	if( modifiedDirection == OCTDIRECTION_SOUTHWEST )	return Vector2( -DIAG, -DIAG );
	if( modifiedDirection == OCTDIRECTION_SOUTH )		return Vector2( +ZERO, -EDGE );
	if( modifiedDirection == OCTDIRECTION_SOUTHEAST )	return Vector2( +DIAG, -DIAG );

	return Vector2::ZERO;
}



