//-------------------------------------------------------------------------------------------------
// TheGame.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.h"
#include "TheGame.h"
#include "Player.h"
#include "Enemy.h"
#include "Base.h"
#include "Projectiles.h"
#include "LineSegment2.h"
#include "GameBalance.h"
#include "BaseCommon.h"
using namespace std;


//-------------------------------------------------------------------------------------------------
// Globals
//-------------------------------------------------------------------------------------------------
TheGame* g_game = NULL;
float testVoob = 0.0f; // this is a test variable that pulses between 0 and 1
bool g_playerIsShooting = false;
bool g_debugSlowFramerate = false;



//ControlScheme g_controlSchemeType = CONTROL_SCHEME_XKOBO;
ControlScheme g_controlSchemeType = CONTROL_SCHEME_SHIPRELATIVEROTATION;



//-------------------------------------------------------------------------------------------------
// Destructor()
//-------------------------------------------------------------------------------------------------
TheGame::~TheGame()
{
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
TheGame::TheGame()
:
m_mouseSensitivity( 1.0f, 1.0f ),
m_shipTurnRate( 0.7f ),
m_zoom( 1.0f ),
m_currentLevel( 0 )
{
}


//-------------------------------------------------------------------------------------------------
// PreInit
//-------------------------------------------------------------------------------------------------
void TheGame::PreInit()
{
	/// Assign application name
	APPLICATION->SetName( "Kobaioh" );

	/// Choose a video mode
//	VIDEO->SetDefaultVidModeProperties( 1280, 960, 32, 85, TheVideoSystem::VIDMODE_FLAG_FULLSCREEN );
	VIDEO->SetDefaultVidModeProperties( 1280, 960, 32, 85, 0 );

	/// Set the mouse in hide & recenter mode
	INPUT->UseWindowsMouseCursor( false );
}


//-------------------------------------------------------------------------------------------------
// PostInit
//-------------------------------------------------------------------------------------------------
void TheGame::PostInit()
{
	RENDERER->ShowFpsToggle();

	/// Load game images
	LoadGameImages();

	g_player.Init();
	StartNewLevel( 0 );
}


//-------------------------------------------------------------------------------------------------
// RunFrame
//-------------------------------------------------------------------------------------------------
void TheGame::RunFrame()
{
	if( g_debugSlowFramerate )
	{
		Sleep( 15 );
	}

	dword_t ms = GetTickCount();
	testVoob = 0.5f + 0.5f * (float) cos( double( ms ) * 0.005f );

	UpdateTimers();
	g_player.Update();
	DrawEverything();
	CheckForCollisions();
	UpdateBases();
	Projectile::UpdateParticles();
	Enemy::UpdateEnemies();
	Enemy::DrawEnemies();
}


//-------------------------------------------------------------------------------------------------
// UpdateTimers
//-------------------------------------------------------------------------------------------------
void TheGame::UpdateTimers()
{
	g_timeNow = APPLICATION->GetTimeFrameBegan();
	g_frameSeconds = APPLICATION->GetFrameSeconds();
}


//-------------------------------------------------------------------------------------------------
// StartNewLevel
//-------------------------------------------------------------------------------------------------
void TheGame::StartNewLevel( int levelNumber )
{
	m_currentLevel = levelNumber;
	InitLevel();
}


//-------------------------------------------------------------------------------------------------
// InitLevel
//-------------------------------------------------------------------------------------------------
void TheGame::InitLevel()
{
	SpawnBasesForThisLevel();
}


//-------------------------------------------------------------------------------------------------
// UpdateBases
//-------------------------------------------------------------------------------------------------
void TheGame::UpdateBases()
{
	list<Base*>::iterator iter;
	for( iter = baseList.begin(); iter != baseList.end(); ++ iter )
	{
		(*iter)->Update();
	}
}


//-------------------------------------------------------------------------------------------------
// SpawnBasesForThisLevel
//-------------------------------------------------------------------------------------------------
void TheGame::SpawnBasesForThisLevel()
{
	float spawnPosX = 200.0f;
	float spawnPosY = 200.0f;
	const int SPAWN_MIN = 15;
	const int SPAWN_MAX = 30;
	int spawnSizeX = SPAWN_MIN;
	int spawnSizeY = SPAWN_MIN;

	float BASE_SPACING = 1500.0f;

	for( spawnPosX = -2000.0f; spawnPosX < 2000.0f; spawnPosX += BASE_SPACING )
	for( spawnPosY = -2000.0f; spawnPosY < 2000.0f; spawnPosY += BASE_SPACING )
	{
		spawnSizeX += 2;
		if( spawnSizeX > SPAWN_MAX )
		{
			spawnSizeX = SPAWN_MIN;
			spawnSizeY += 2;
			if( spawnSizeY > SPAWN_MAX )
			{
				spawnSizeY = SPAWN_MIN;
			}
		}

		GenerateBase( spawnPosX, spawnPosY, spawnSizeX, spawnSizeY );
	}


/*
	Vector2 position( 200.0f, -300.0f );
	Base& base = Base::SpawnBase( position, 5, 4 );
	base.AddHeart( 2, 1 );
	base.AddConnection( 0, 0, 0, 1 );
	base.AddConnection( 0, 1, 1, 1 );
	base.AddConnection( 1, 1, 2, 1 );
	base.AddConnection( 2, 1, 1, 2 );
	base.AddConnection( 1, 2, 0, 2 );

	Vector2 position2( 0.0f, 400.0f );
	Base& base2 = Base::SpawnBase( position2, 5, 4 );
	base2.AddHeart( 2, 1 );
	base2.AddConnection( 0, 0, 0, 1 );
	base2.AddConnection( 0, 1, 1, 1 );
	base2.AddConnection( 1, 1, 2, 1 );
	base2.AddConnection( 2, 1, 1, 2 );
	base2.AddConnection( 1, 2, 0, 2 );

	Vector2 position3( 400.0f, 500.0f );
	Base& base3 = Base::SpawnBase( position3, 5, 4 );
	base3.AddHeart( 2, 1 );
	base3.AddConnection( 0, 0, 0, 1 );
	base3.AddConnection( 0, 1, 1, 1 );
	base3.AddConnection( 1, 1, 2, 1 );
	base3.AddConnection( 2, 1, 1, 2 );
	base3.AddConnection( 1, 2, 0, 2 );
	base3.AddConnection( 1, 0, 1, 1 );
	base3.AddConnection( 1, 0, 2, 0 );
	base3.AddConnection( 2, 1, 3, 0 );
	base3.AddConnection( 3, 1, 3, 0 );
*/
//	Vector2 position4( 500.0f, 0.0f );
//	Base& base4 = Base::SpawnBase( position4, 15, 15 );
//	base4.Generate();

//	base4.AddHeart( 2, 1 );
//	base4.AddConnection( 0, 0, 0, 1 );
//	base4.AddConnection( 0, 1, 1, 1 );
//	base4.AddConnection( 1, 1, 2, 1 );
//	base4.AddConnection( 2, 1, 1, 2 );
//	base4.AddConnection( 1, 2, 0, 2 );
}


//-------------------------------------------------------------------------------------------------
// GenerateBase
//-------------------------------------------------------------------------------------------------
void TheGame::GenerateBase( float posX, float posY, int sizeX, int sizeY )
{
	Vector2 position( posX, posY );
	Base& base = Base::SpawnBase( position, sizeX, sizeY );
	base.Generate();
}


//-------------------------------------------------------------------------------------------------
// OnKeyDown
//-------------------------------------------------------------------------------------------------
void TheGame::OnKeyDown( const byte_t key )
{
	CONSOLE->Printf( "OnKeyDown: #%d = 0x%4x = '%c'\n", key, key, key );

	if( key == VK_ESCAPE )
	{
		APPLICATION->RequestShutdown();
	}
	else if( key == VK_F11 )
	{
		RENDERER->ShowFpsToggle();
	}
	else if( key == VK_TILDE )
	{
		RENDERER->ShowConsoleToggle();
	}
/*	else if( key == 'E' || key == 'D' || key == 'S' || key == 'F' )
	{
		OnMoveKeyEvent( key, true );
	}
*/	else if( IsKeyANewMoveKey( key ) )
	{
		OnNewMoveKeyEvent( key, true );
	}
	else if( key == VK_LBUTTON || key == VK_SPACE )
	{
		g_playerIsShooting = true;
	}
	else if( key == VK_F9 )
	{
		g_debugDrawCollisions = !g_debugDrawCollisions;
	}
	else if( key == VK_F8 )
	{
		g_debugSlowFramerate = !g_debugSlowFramerate;
	}
}


//-------------------------------------------------------------------------------------------------
// OnKeyUp
//-------------------------------------------------------------------------------------------------
void TheGame::OnKeyUp( const byte_t key )
{
	if( key == 'E' || key == 'D' || key == 'S' || key == 'F' )
	{
		OnMoveKeyEvent( key, false );
	}
	else if( IsKeyANewMoveKey( key ) )
	{
		OnNewMoveKeyEvent( key, false );
	}
	else if( key == VK_LBUTTON || key == VK_SPACE )
	{
		g_playerIsShooting = false;
	}
}


//-------------------------------------------------------------------------------------------------
// OnMoveKeyEvent
//-------------------------------------------------------------------------------------------------
void TheGame::OnMoveKeyEvent( byte_t key, bool isKeyDownEvent )
{
	if( key == 'E' )
	{
		g_player.m_isKeyDown_Up = isKeyDownEvent;
	}
	else if( key == 'D' )
	{
		g_player.m_isKeyDown_Down = isKeyDownEvent;
	}
	else if( key == 'S' )
	{
		g_player.m_isKeyDown_Left = isKeyDownEvent;
	}
	else if( key == 'F' )
	{
		g_player.m_isKeyDown_Right = isKeyDownEvent;
	}
}


//-------------------------------------------------------------------------------------------------
// OnNewMoveKeyEvent
//-------------------------------------------------------------------------------------------------
void TheGame::OnNewMoveKeyEvent( byte_t key, bool isKeyDownEvent )
{
	if( g_controlSchemeType == CONTROL_SCHEME_XKOBO )
	{
		OnNewMoveKeyEvent_XKobo( key, isKeyDownEvent );
	}
	else if( g_controlSchemeType == CONTROL_SCHEME_SHIPRELATIVEROTATION )
	{
		OnNewMoveKeyEvent_ShipRelativeRotation( key, isKeyDownEvent );
	}
}


//-------------------------------------------------------------------------------------------------
// OnNewMoveKeyEvent_XKobo
//-------------------------------------------------------------------------------------------------
void TheGame::OnNewMoveKeyEvent_XKobo( byte_t key, bool isKeyDownEvent )
{
	/// Clear all key state variables; we don't use 'em in this mode
	g_player.m_isKeyDown_Up = false;
	g_player.m_isKeyDown_Down = false;
	g_player.m_isKeyDown_Left = false;
	g_player.m_isKeyDown_Right = false;

	if( !isKeyDownEvent )
		return;

	if( IsKeyANewMoveKey_Up( key ) )
	{
		g_player.m_direction = OCTDIRECTION_NORTH;
	}
	else if( IsKeyANewMoveKey_Down( key ) )
	{
		g_player.m_direction = OCTDIRECTION_SOUTH;
	}
	else if( IsKeyANewMoveKey_Left( key ) )
	{
		g_player.m_direction = OCTDIRECTION_WEST;
	}
	else if( IsKeyANewMoveKey_Right( key ) )
	{
		g_player.m_direction = OCTDIRECTION_EAST;
	}
	else if( IsKeyANewMoveKey_UpRight( key ) )
	{
		g_player.m_direction = OCTDIRECTION_NORTHEAST;
	}
	else if( IsKeyANewMoveKey_UpLeft( key ) )
	{
		g_player.m_direction = OCTDIRECTION_NORTHWEST;
	}
	else if( IsKeyANewMoveKey_DownLeft( key ) )
	{
		g_player.m_direction = OCTDIRECTION_SOUTHWEST;
	}
	else if( IsKeyANewMoveKey_DownRight( key ) )
	{
		g_player.m_direction = OCTDIRECTION_SOUTHEAST;
	}
}



//-------------------------------------------------------------------------------------------------
// OnNewMoveKeyEvent_ShipRelativeRotation
//-------------------------------------------------------------------------------------------------
void TheGame::OnNewMoveKeyEvent_ShipRelativeRotation( byte_t key, bool isKeyDownEvent )
{
	if( IsKeyANewMoveKey_Up( key ) )
	{
		g_player.m_isKeyDown_Up = isKeyDownEvent;
	}
	else if( IsKeyANewMoveKey_Down( key ) )
	{
		g_player.m_isKeyDown_Down = isKeyDownEvent;
	}
	else if( IsKeyANewMoveKey_Left( key ) )
	{
		g_player.m_isKeyDown_Left = isKeyDownEvent;
	}
	else if( IsKeyANewMoveKey_Right( key ) )
	{
		g_player.m_isKeyDown_Right = isKeyDownEvent;
	}
}




byte_t g_keysMoveUp[]			= { VK_NUMPAD8, VK_UP, 'E' };
byte_t g_keysMoveDown[]			= { VK_NUMPAD2, VK_DOWN, 'D' };
byte_t g_keysMoveLeft[]			= { VK_NUMPAD4, VK_LEFT, 'S' };
byte_t g_keysMoveRight[]		= { VK_NUMPAD6, VK_RIGHT, 'F' };

byte_t g_keysMoveUpLeft[]		= { VK_NUMPAD7, VK_HOME };
byte_t g_keysMoveUpRight[]		= { VK_NUMPAD9, VK_PRIOR };
byte_t g_keysMoveDownLeft[]		= { VK_NUMPAD1, VK_END };
byte_t g_keysMoveDownRight[]	= { VK_NUMPAD3, VK_NEXT };


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey( byte_t key )
{
	if( IsKeyANewMoveKey_Up( key ) )
		return true;

	if( IsKeyANewMoveKey_Down( key ) )
		return true;

	if( IsKeyANewMoveKey_Left( key ) )
		return true;

	if( IsKeyANewMoveKey_Right( key ) )
		return true;

	if( IsKeyANewMoveKey_UpRight( key ) )
		return true;

	if( IsKeyANewMoveKey_DownRight( key ) )
		return true;

	if( IsKeyANewMoveKey_UpLeft( key ) )
		return true;

	if( IsKeyANewMoveKey_DownLeft( key ) )
		return true;

	return false;
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_Up
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_Up( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveUp ) / sizeof( g_keysMoveUp[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveUp, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_Down
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_Down( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveDown ) / sizeof( g_keysMoveDown[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveDown, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_Left
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_Left( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveLeft ) / sizeof( g_keysMoveLeft[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveLeft, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_Right
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_Right( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveRight ) / sizeof( g_keysMoveRight[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveRight, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_UpRight
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_UpRight( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveUpRight ) / sizeof( g_keysMoveUpRight[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveUpRight, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_UpLeft
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_UpLeft( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveUpLeft ) / sizeof( g_keysMoveUpLeft[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveUpLeft, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_DownRight
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_DownRight( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveDownRight ) / sizeof( g_keysMoveDownRight[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveDownRight, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_DownLeft
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_DownLeft( byte_t key )
{
	int numEntriesInList = sizeof( g_keysMoveDownLeft ) / sizeof( g_keysMoveDownLeft[0] );
	return IsKeyANewMoveKey_List( key, g_keysMoveDownLeft, numEntriesInList );
}


//-------------------------------------------------------------------------------------------------
// IsKeyANewMoveKey_List
//-------------------------------------------------------------------------------------------------
bool TheGame::IsKeyANewMoveKey_List( byte_t key, byte_t* keyList, int listSize )
{
	for( int i = 0; i < listSize; ++ i )
	{
		if( keyList[ i ] == key )
			return true;
	}

	return false;
}

