//-------------------------------------------------------------------------------------------------
// TheGame.h (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheGame_
#define _include_TheGame_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
//#include "pch.h"
#include "gameCommon.h"
#include "Projectiles.h"
#include "Base.h"


enum ControlScheme
{
	CONTROL_SCHEME_XKOBO,
	CONTROL_SCHEME_SHIPRELATIVEROTATION,
};



///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheGame
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheGame
{
private:
public:
	Vector2		m_mouseSensitivity;
	float		m_shipTurnRate;
	float		m_zoom;
	int			m_currentLevel;

	/// Constructors / destructors
	TheGame(); // default constructor
	~TheGame(); // destructor

	/// Public methods
	void PreInit();
	void PostInit();
	void RunFrame();

	void OnKeyDown( const byte_t key );
	void OnKeyUp( const byte_t key );
	void OnMoveKeyEvent( byte_t key, bool isKeyDownEvent );

	/// TheGame.cpp
	void StartNewLevel( int levelNumber );
	void InitLevel();
	void SpawnBasesForThisLevel();
	void UpdateBases();
	void GenerateBase( float posX, float posY, int sizeX, int sizeY );

	void OnNewMoveKeyEvent( byte_t key, bool isKeyDownEvent );
	bool IsKeyANewMoveKey( byte_t key );
	bool IsKeyANewMoveKey_Up( byte_t key );
	bool IsKeyANewMoveKey_Down( byte_t key );
	bool IsKeyANewMoveKey_Left( byte_t key );
	bool IsKeyANewMoveKey_Right( byte_t key );
	bool IsKeyANewMoveKey_UpRight( byte_t key );
	bool IsKeyANewMoveKey_UpLeft( byte_t key );
	bool IsKeyANewMoveKey_DownRight( byte_t key );
	bool IsKeyANewMoveKey_DownLeft( byte_t key );
	bool IsKeyANewMoveKey_List( byte_t key, byte_t* keyList, int listSize );
	void OnNewMoveKeyEvent_XKobo( byte_t key, bool isKeyDownEvent );
	void OnNewMoveKeyEvent_ShipRelativeRotation( byte_t key, bool isKeyDownEvent );
	void UpdateTimers();

	/// TheGame_Physics.cpp
	void ResolveCollisionBulletVsNode( Base& base, Projectile& bullet, BaseNode& node );
	void ResolveCollisionBulletVsPipe( Base& base, Projectile& bullet, BasePipe& pipe, LineSegment2& pipeSegment );
	void CheckForCollisions();
	void CheckForBulletCollisionsVersesBase( Base& base, const AABB2& bulletBounds, int particleType );
	bool CheckForCollisionBulletVsBase( Base& base, Projectile& bullet );
	void CheckForCollisionBulletVsPipe( Base& base, Projectile& bullet, BaseNode& node, int x, int y, HexDirection direction );
	bool CheckAndResolveCollisionBulletVsPipe( Base& base, Projectile& bullet, BaseNode& node, HexDirection direction );
	AABB2 BuildPlayerBulletBounds();

	/// TheGame_Draw.cpp
	void LoadGameImages();
	void DrawEverything();
	void DrawGameField();
	void DrawGUI();
	void DrawStarField();
	void DrawBases();
	void DrawProjectiles();
	void DrawProjectile( const Projectile& p, int particleType );
	void SetGameFieldCoordinateSystemUnRotated();
	void RotateGameFieldCoordinateSystem();
};

//-------------------------------------------------------------------------------------------------
// Externally viewable globals
//-------------------------------------------------------------------------------------------------
/// From TheGame.cpp
extern TheGame* g_game;
extern float testVoob;
extern ControlScheme g_controlSchemeType;
extern bool g_playerIsShooting;


/// From TheGame_Physics.cpp
extern bool g_debugDrawCollisions;



#endif // _include_TheGame_
