// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.h"
#include "TheGame.h"
#include "Image.h"
#include "Player.h"
#include "Base.h"
#include "Capsule2.h"
#include "AABB2.h"
#include "Projectiles.h"
#include "LineSegment2.h"
using namespace std;


float moveParm = 0.0f;


//-------------------------------------------------------------------------------------------------
// SetGameFieldCoordinateSystemUnRotated
//-------------------------------------------------------------------------------------------------
void TheGame::SetGameFieldCoordinateSystemUnRotated()
{
	const Vector2 bottomLeft( 0.0f, 0.0f );
	const Vector2 topRight( 1280.0f, 960.0f );
	const Vector2 screenCenter = 0.5f * (topRight + bottomLeft) + Vector2( 0.0f, -80.0f );
	const Vector2 scale( m_zoom, m_zoom );

	RENDERER->SetCoordinateSystem( bottomLeft, topRight );
	RENDERER->TranslateCoordinateSystem( g_player.m_pos, screenCenter );
	RENDERER->ScaleCoordinateSystem( scale, g_player.m_pos );
	glPointSize( m_zoom );
}


//-------------------------------------------------------------------------------------------------
// RotateGameFieldCoordinateSystem
//-------------------------------------------------------------------------------------------------
void TheGame::RotateGameFieldCoordinateSystem()
{
	float rotationRadians = -g_player.m_facingRadians + float( HALF_PI );
	RENDERER->RotateCoordinateSystem( rotationRadians, g_player.m_pos );
}


//-------------------------------------------------------------------------------------------------
// DrawStarField
//-------------------------------------------------------------------------------------------------
void TheGame::DrawStarField()
{
	glPointSize( m_zoom * 4.0f );
	glEnable( GL_POINT_SMOOTH );
	glBegin( GL_POINTS );
	{
		glVertex2f( -100.0f, -300.0f );
		glVertex2f( +200.0f, -500.0f );
		glVertex2f( -300.0f, +200.0f );
		glVertex2f( -400.0f, +100.0f );
		glVertex2f( +500.0f, -400.0f );
		glVertex2f( +100.0f, +300.0f );
		glVertex2f( +200.0f, -500.0f );
		glVertex2f( -300.0f, +200.0f );
		glVertex2f( +400.0f, -100.0f );
		glVertex2f( -500.0f, +400.0f );

		glVertex2f( -400.0f, -100.0f );
		glVertex2f( +600.0f, -200.0f );
		glVertex2f( -100.0f, +100.0f );
		glVertex2f( -200.0f, +200.0f );
		glVertex2f( +400.0f, -600.0f );
		glVertex2f( +000.0f, +300.0f );
		glVertex2f( +300.0f, -200.0f );
		glVertex2f( -600.0f, +400.0f );
		glVertex2f( +200.0f, -300.0f );
		glVertex2f( -400.0f, +100.0f );

		glVertex2f( -50.0f, -150.0f );
		glVertex2f( +100.0f, -250.0f );
		glVertex2f( -150.0f, +100.0f );
		glVertex2f( -200.0f, +50.0f );
		glVertex2f( +250.0f, -200.0f );
		glVertex2f( +50.0f, +150.0f );
		glVertex2f( +100.0f, -250.0f );
		glVertex2f( -150.0f, +100.0f );
		glVertex2f( +200.0f, -50.0f );
		glVertex2f( -250.0f, +200.0f );
	}
	glEnd();
}


//-------------------------------------------------------------------------------------------------
// DrawProjectile
//-------------------------------------------------------------------------------------------------
void TheGame::DrawProjectile( const Projectile& p, int particleType )
{
	if( g_debugDrawCollisions )
	{
		glLineWidth( 4.0f );
		RENDERER->DrawVector( p.m_pos, p.m_endPos, Rgba::BLACK, Rgba::ORANGE );
		glLineWidth( 1.0f );
	}
	else
	{
		Image::Get( IMAGE_PLAYER_GEM_RED ).Draw( p.m_pos, p.m_density * 0.5f );
	}
}


//-------------------------------------------------------------------------------------------------
// DrawProjectiles
//-------------------------------------------------------------------------------------------------
void TheGame::DrawProjectiles()
{
	/// For each particle type...
	for( int particleType = 0; particleType < PROJECTILETYPE_NUM_TYPES; ++ particleType )
	{
		/// Draw each live projectile of this type
		list<Projectile*>::iterator iter;
		for( iter = projectileList[ particleType ].begin(); iter != projectileList[ particleType ].end(); ++ iter )
		{
			Projectile* p = *iter;
			DrawProjectile( *p, particleType );
		}
	}
}


//-------------------------------------------------------------------------------------------------
// DrawBases
//-------------------------------------------------------------------------------------------------
void TheGame::DrawBases()
{
	list<Base*>::iterator iter;
	for( iter = baseList.begin(); iter != baseList.end(); ++ iter )
	{
		Base* base = *iter;
		base->Draw();
	}
}


//-------------------------------------------------------------------------------------------------
// DrawGameField
//-------------------------------------------------------------------------------------------------
void TheGame::DrawGameField()
{
	/// Draw rotated game field elements (stars, enemies, etc)
	SetGameFieldCoordinateSystemUnRotated(); // set up base background gameplay-field coordinate system
	RotateGameFieldCoordinateSystem(); // set up base background gameplay-field coordinate system

	/// Draw the "aiming beam"
//	Vector2 end = g_player.pos + (1000.0f * g_player.forward);
//	Rgba startColor( 1.0f, 1.0f, 1.0f, 0.3f );
//	Rgba endColor( 1.0f, 1.0f, 1.0f, 0.0f );
//	RENDERER->DrawVector( g_player.pos, end, startColor, endColor );

	/// Draw starfield
	RENDERER->PushCoordinateSystem();
	RENDERER->SetColor( 1.0f, 1.0f, 1.0f, 0.1f );
	RENDERER->ScaleCoordinateSystem( 0.5f, g_player.m_pos );
	DrawStarField();
	RENDERER->PopCoordinateSystem();

	/// Draw bullets
	DrawProjectiles();

	/// Draw bases
	DrawBases();

	/// Draw enemies
	Image::Get( IMAGE_ENEMY_SHIP_RED ).Draw( Vector2( 600.0f, 0.0f ), 2.0f );
	Image::Get( IMAGE_ENEMY_SHIP_RED ).Draw( Vector2( 300.0f, 300.0f ), 2.0f );
	Image::Get( IMAGE_ENEMY_SHIP_RED ).Draw( Vector2( 100.0f, -400.0f ), 2.0f );
	Image::Get( IMAGE_ENEMY_SHIP_RED ).Draw( Vector2( 200.0f, 100.0f ), 2.0f );
	Image::Get( IMAGE_ENEMY_SHIP_RED ).Draw( Vector2( 000.0f, -300.0f ), 2.0f );
	Image::Get( IMAGE_ENEMY_SHIP_RED ).Draw( Vector2( 500.0f, 400.0f ), 2.0f );

	/// Draw unrotated game field elements (like the player)
	SetGameFieldCoordinateSystemUnRotated(); // set up base background gameplay-field coordinate system
	g_player.Draw(); // draw the player
	RotateGameFieldCoordinateSystem(); // set up base background gameplay-field coordinate system
}


//-------------------------------------------------------------------------------------------------
// DrawGUI
//-------------------------------------------------------------------------------------------------
void TheGame::DrawGUI()
{
}


//-------------------------------------------------------------------------------------------------
// DrawEverything
//-------------------------------------------------------------------------------------------------
void TheGame::DrawEverything()
{
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	DrawGameField();
	DrawGUI();
}

//-------------------------------------------------------------------------------------------------
// LoadGameImages
//-------------------------------------------------------------------------------------------------
void TheGame::LoadGameImages()
{
	Image::Create( IMAGE_PLAYER_SHIP_BODY,	"data/gfx/tri_shooter.tga" ).CreateTexture();
	Image::Create( IMAGE_PLAYER_GEM_RED,	"data/gfx/tri_redgem.tga" ).CreateTexture();

	Image::Create( IMAGE_ENEMY_SHIP_RED,	"data/gfx/enemy_r.tga" ).CreateTexture();
	Image::Create( IMAGE_ENEMY_SHIP_GREEN,	"data/gfx/enemy_g.tga" ).CreateTexture();
	Image::Create( IMAGE_ENEMY_SHIP_BLUE,	"data/gfx/enemy_b.tga" ).CreateTexture();
}

